/*
 * $Id: tcp.c,v 1.3 1998/05/19 23:53:13 garrett Exp $
 *
 * Basic TCP services.
 *
 * Written by Garrett D'Amore <garrett@yavin.org>.
 *
 * Copyright (C) 1997-1998 Garrett D'Amore.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>

#ifndef HAVE_STRERROR
extern char *sys_errlist[];
#define strerror(x) (sys_errlist[x])
#endif

static char *tcpError = "";

const char *tcpStrError(void)
{
    return tcpError;
}

void tcpClearError(void)
{
    tcpError = "";
}

short tcpService(const char *service)
{
    struct servent	*se;
    short		port = -1;

    /* get port number */
    se = getservbyname(service, "tcp");
    if (se)
    {
	/* it was in the services database */
	port = se->s_port;
    }
    else
    {
	/* maybe its a number */
	port = htons(atoi(service));
    }
    if (port < 1)
    {
	tcpError = "Unknown service.";
	port = -1;
    }
    
    return port;
}

int tcpConnect(const char *host, short port)
{
    struct sockaddr_in	server;
    struct hostent	*he;
    int			sd;

    assert(host);
	
    /* clear out structure */
    memset((char *)&server, 0, sizeof(server));

    /* flesh out sockaddr_in structs */
    server.sin_family = AF_INET;
    server.sin_port = port;

    /* try hostname first */
    he = gethostbyname(host);

    if (he)
    {
	memcpy((char *)&server.sin_addr.s_addr, he->h_addr, he->h_length);
    }
    else
    {
	/* try as IP address if not a host name */
	server.sin_addr.s_addr = inet_addr(host);
	if (server.sin_addr.s_addr == -1)
	{
	    tcpError = "Unknown host.";
	    return -1;
	}
    }
    
    /* create the stream socket */
    sd = socket(AF_INET, SOCK_STREAM, 0);
    if (sd == -1)
    {
	tcpError = strerror(errno);
	return -1;
    }

    /* connect! */
    if (connect(sd, (struct sockaddr *) &server, sizeof(server)) == -1)
    {
	tcpError = strerror(errno);
	close(sd);
	return -1;
    }

    return sd;
}
